package be.iminds.ilabt.jfed.experimenter_gui.ui;

import be.iminds.ilabt.jfed.experimenter_gui.config.TestbedSiteInfo;
import be.iminds.ilabt.jfed.experimenter_gui.util.StringConverters;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.MaintenanceInfo;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.ResourceClass;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.ServerGlimpse;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import javafx.geometry.Insets;
import javafx.scene.control.Label;
import javafx.scene.control.ListCell;
import javafx.scene.control.Tooltip;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.Priority;
import javafx.scene.paint.Color;
import org.controlsfx.glyphfont.FontAwesome;
import org.controlsfx.glyphfont.Glyph;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.time.ZonedDateTime;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

public class TestbedSiteCell extends ListCell<TestbedSiteInfo> {
    private static final Logger LOG = LoggerFactory.getLogger(TestbedSiteCell.class);

    private static final Comparator<? super MaintenanceInfo> MAINTENANCE_MESSAGE_COMPARATOR =
            Comparator.nullsLast(Comparator.comparing(MaintenanceInfo::getStart))
                    .thenComparing(Comparator.nullsLast(Comparator.comparing(MaintenanceInfo::getEnd)))
                    .thenComparing(MaintenanceInfo::getMessage);


    protected final HBox container = new HBox(2);

    protected final Label nameLabel = new Label("");

    @Nonnull
    protected final AvailabilityMeter availabilityMeter;

    protected boolean debug = false;

    public TestbedSiteCell() {
        Pane fillerPane = new Pane();
        HBox.setHgrow(fillerPane, Priority.ALWAYS);

        availabilityMeter = new AvailabilityMeter();

        container.setPadding(new Insets(0, 5, 2, 0));
        container.getChildren().add(nameLabel);
        container.getChildren().add(availabilityMeter);

        setGraphic(container);
    }


    @Override
    protected void updateItem(TestbedSiteInfo item, boolean empty) {
        if (debug)
            LOG.debug("TestbedSiteCell.updateItem hrn="+(item == null ? null : item.getHrn())+" empty="+empty);
        super.updateItem(item, empty);
        if (!empty && item != null) {
            nameLabel.setText(item.getHrn());

            if (item.getAvailableComponents() == -1 || item.getAvailableComponents() == -1) {
                availabilityMeter.setVisible(false);
//                availabilityMeter.setVisible(true);
//                availabilityMeter.setUndetermined(true);
//
//                Tooltip.install(availabilityMeter,
//                        new Tooltip("number of resources unknown"));
            } else {
                availabilityMeter.setAvailable(item.getAvailableComponents()); //todo: use property
                availabilityMeter.setTotal(item.getTotalComponents());
                availabilityMeter.setVisible(true);
                availabilityMeter.setUndetermined(false);

                Tooltip.install(availabilityMeter,
                        new Tooltip(item.getAvailableComponents() + " of "
                                + item.getTotalComponents() + " resources available"));
            }
        } else {
            nameLabel.setText("");
            availabilityMeter.setVisible(false);
        }
    }

}
